$title	DICE version 1994 -- a mixed complementarity formulation

scalar nlpsolve /0/;

*	Start out by solving the NLP model:

$include dice94

variables
	PY(t)		Output
	PQ(t)		Underlying production function
	PC(t)		Consumption
	PA(t)		Shadow price on abatement cost coefficent
	PD(t)		shadow price on damage coefficent
	PK(t)		Capital balance
	PKC(t)		Terminal condition of K
	PE(t)		Emissions process
	PFORC(t)	Radiative forcing equation
	PM(t)		CO2 distribution equation
	PTE(t)		Temperature-climate equation for atmosphere
	PTL(t)		Temperature-climate equation for lower oceans;

EQUATIONS
	EQ_C(t)		Consumption trillion US dollars
	EQ_K(t)		Capital stock trillion US dollars
	EQ_I(t)		Investment trillion US dollars
	EQ_Y(t)		Output net abatement and damage costs 
	EQ_Q(t)		Gross Output 
	EQ_A(t)		Abatement
	EQ_D(t)		Damage
	EQ_E(t)		CO2-equiv emissions billion t
	EQ_M(t)		CO2-equiv concentration billion t
	EQ_MIU(t)	Emission control rate GHGs
	EQ_FORC(t)	Radiative forcing - W per m2
	EQ_TE(t)	Temperature - atmosphere C
	EQ_TL(t)	Temperature - lower ocean C;

EQ_C(t)..	10 * rr(t) * L(t) / (0.55*C(t)) =E= PC(t);
EQ_K(t)..	K(t) * (PK(t) + (PKC(t)*dk-rr(t)*phik)$tlast(t)) =G=
		gamma * PQ(t) * Q(t) + PK(t+1) * (1-dk)**10 * K(t);
EQ_I(t)..	PC(t) =E= 10 * PK(t+1) + PKC(t)$tlast(t);
EQ_Y(t)..	PY(t) * (1+D(t)) =E= PC(t);
EQ_Q(t)..	PQ(t) =E= PY(t) * (1 - A(t)) + PE(t) * 10 * sigma(t) * (1-MIU(t));
EQ_A(t)..	PA(t) + PY(t)*Q(t) =E= 0;
EQ_D(t)..	PD(t) + PY(t)*Y(t) =E= 0;
EQ_MIU(t)..	PE(t)*10*sigma(t)*Q(t) =E= PA(t) * b1 * b2 * MIU(t)**(b2-1);
EQ_E(t)..	PE(t) =E= PM(t)*atret;
EQ_M(t)..	PM(t) =E= (1-deltam)*PM(t+1) + (rr(t)*phim)$tlast(t) + PFORC(t)*4.1/(LOG(2)*M(t));
EQ_FORC(t)..	PFORC(t) =E= PTE(t+1)*c1;
EQ_TE(t)..	PTE(t) + c1*(lam+c3)*PTE(T+1) 
		  =E= PTE(t+1) + c4*PTL(t+1) + PD(t)*2*(A1/9)*TE(t) + (rr(t)*phite)$tlast(t);
EQ_TL(t)..	PTL(t) =E= PTL(t+1)*(1-c4) + c1*c3*PTE(t+1)

MODEL DICEMCP /
	YY.PY, QQ.PQ, CC.PC, KK.PK, KC.PKC, EE.PE, FORCE.PFORC, MM.PM,
	TTE.PTE, TLE.PTL, EQ_C.C, EQ_K.K, EQ_I.I, EQ_Y.Y, EQ_Q.Q, EQ_E.E,
	EQ_D.D, EQ_A.A, AA.PA, DD.PD,
	EQ_M.M, EQ_MIU.MIU, EQ_FORC.FORC, EQ_TE.TE, EQ_TL.TL /;

*	Verify that the NLP solution is consistent with the MCP solution:

PY.L(t) = YY.M(t);
PQ.L(t) = QQ.M(t);
PC.L(t) = CC.M(t);
PK.L(t) =KK.M(t);
PA.L(t) = AA.M(t);
PD.L(t) = DD.M(t);
PKC.L(t) = KC.M(t);
PE.L(t) = EE.M(t);
PFORC.L(t) = FORCE.M(t);
PM.L(t) = MM.M(t);
PTE.L(t) = TTE.M(t);
PTL.L(t) =TLE.M(t);
DICEMCP.ITERLIM = 0;
SOLVE DICEMCP USING MCP;
