$TITLE  DICE version 1994 -- MCP implementation with taxes

$if not set tk0 $set tk0 0.25

scalar  tk0     Baseline capital tax rate /%tk0%/;

scalar	g0	Baseline government /1/;

$if not set tc $set tc 60
$if not set t  $set t  40

set	tc	/1*%tc%/,
	t(tc)	/1*%t%/;

$include dicedata

alias (t,tp); alias (tp,tpp);

PARAMETER
	teref(tc)	Reference values of temperature
	pdref(tc)	Reference present value of damage
	grad(tc,t)	Local dependence of D(tp) on E(t),
	eref(tc)	Reference values of emissions
	xi(t)		Post-terminal damage value;

VARIABLES
	C(tc)		Consumption trillion US dollars
	G		Government demand
	K(t)		Capital stock trillion US dollars
	I(t)		Investment trillion US dollars
	Y(t)		Output net abatement and damage costs 
	D(tc)		Damage
	A(t)		Abatement cost
	Q(t)		Gross Output 
	E(t)		CO2-equiv emissions billion t
	MIU(t)		Emission control rate GHGs

	PY(t)		Output
	PQ(t)		Underlying production function
	PC(t)		Consumption
	PA(t)		Shadow price on abatement cost coefficent
	PD(t)		shadow price on damage coefficent
	PK(t)		Capital balance
	PE(t)		Emissions process
	TK		Capital tax rate 
	KT		Terminal Capital stock,
	PKT		Shadow price on terminal capital
	RENT(t)		Rents on emission permits;

POSITIVE VARIABLES MIU, Y, C, K, I, PE;

EQUATIONS
	YY(t)		Output,
	AA(t)		Abatement,
	DD(t)		Damage (linear climate model),
	QQ(t)		Underlying production function,
	CC(t)		Consumption,
	KK(t)		Capital balance,
	EE(t)		Emissions process,

	EQ_C(t)		Consumption trillion US dollars,
	EQ_K(t)		Capital stock trillion US dollars,
	EQ_I(t)		Investment trillion US dollars,
	EQ_Y(t)		Output net abatement and damage costs,
	EQ_Q(t)		Gross Output,
	EQ_A(t)		Abatement,
	EQ_D(t)		Damage,
	EQ_MIU(t)	Emission control rate GHGs
	EQ_E(t)		CO2-equiv emissions billion t
	EQ_PKT		Equilibrium for terminal capital market,
	EQ_KT		Equilibrium for terminal capital stock
	EQ_G		Government budget,
	EQ_TK		Capital tax rate
	EQ_RENT(t)	Rental rate;



CC(t)..	        C(t) + G * L(t)/L0 =E= Y(t) - I(t);
YY(t)..	        Y(t) =E= Q(t)*(1-A(t)) - D(t)*Y(t);
AA(t)..	        A(t) =E=  b1 * MIU(t)**b2;
QQ(t)..	        Q(t) =E= al(t) * L(t)**(1-gamma) * K(t)**gamma;
KK(t)..	        K(t) =L= (1-dk)**10 * K(t-1) + 10 * I(t-1) + (k0*0.9)$tfirst(t);
EE(t)..	        E(t) =G= 10 * sigma(t) * (1-MIU(t)) * Q(t);
DD(t)..	        D(t) =E= (a1/9)*SQR(teref(t) + sum(tp, grad(t,tp)*(E(tp)-eref(tp))));
EQ_C(t)..       10 * rr(t) * L(t) / (0.55*C(t)) =E= PC(t);
EQ_K(t)..       K(t) * PK(t)  =G=
		  gamma*PQ(t)*Q(t)/(1+TK) + (PK(t+1)+PKT$tlast(t)) * (1-dk)**10 * K(t);
EQ_I(t)..       PC(t) =E= 10 * (PK(t+1) + PKT$tlast(t));
EQ_Y(t)..       PY(t) * (1+D(t)) =E= PC(t);
EQ_Q(t)..       PQ(t) =E= PY(t)*(1-A(t)) - PE(t)*10*sigma(t)*(1-MIU(t));
EQ_E(t)..       -PE(t) =E= SUM(tp, PD(tp)*grad(tp,t)*2*(a1/9) *
			(teref(tp) + sum(tpp, grad(t,tpp)*(E(tpp)-eref(tpp))))) 
			+ SUM(tlast, PD(tlast)*xi(t));
EQ_A(t)..       PA(t) + PY(t)*Q(t) =E= 0;
EQ_D(t)..       PD(t) + PY(t)*Y(t) =E= 0;
EQ_MIU(t)..	-PE(t)*10*sigma(t)*Q(t) =E= PA(t) * b1 * b2 * MIU(t)**(b2-1);
EQ_PKT..	SUM(tlast, 10 * I(tlast) + K(tlast) * (1-dk/100)**10) =E= KT;
EQ_KT..		SUM(tlast(t), I(t)/I(t-1) - Y(t)/Y(t-1)) =E= 0;
EQ_TK..		G =e= g0;
EQ_G..		G * SUM(t,L(t)/L0*PC(t)) =E= 
		TK * sum(t, gamma*PQ(t)*Q(t)/(1+TK)) + SUM(t, RENT(t));
EQ_RENT(t)..	RENT(t) =e= PE(t)*E(t) - PY(t)*A(t)*Q(t);

MODEL DICEMCP /CC.PC, YY.PY, AA.PA, QQ.PQ, KK.PK, EE.PE, DD.PD, EQ_C.C,
     EQ_K.K, EQ_I.I, EQ_Y.Y, EQ_Q.Q, EQ_E.E, EQ_A.A, EQ_D.D, EQ_MIU.MIU, 
     EQ_KT.KT, EQ_PKT.PKT, EQ_G.G, EQ_TK.TK, EQ_RENT.RENT /;

PARAMETERS
	m(tc)		CO2-equiv concentration billion t,
	forc(tc)	Radiative forcing - W per m2,
	te(tc)		Temperature - atmosphere C,
	tl(tc)		Temperature - lower ocean C,
	deltaE		Difference iterval /0.01/;

m(tfirst) = M0;
te(tfirst) = T0;
tl(tfirst) = TL0;

parameter	teinit(tc)	Tracking of initial temperature trajectory;

*       Generate GAMS code for the climate model:

$onecho >climatemodel.gms
$onuni
loop(tc,
	m(tc)	 = 590 + atret*eref(tc) + (1-deltam)*(m(tc-1)-590) + m0$tfirst(tc);
	forc(tc) = 4.1*(LOG(m(tc)/590)/LOG(2)) + forcoth(tc);
	te(tc) = te(tc-1)+c1*(forc(tc-1)-lam*te(tc-1)-c3*(te(tc-1)-tl(tc-1))) + t0$tfirst(tc);
	tl(tc) = tl(tc-1)+c4*(te(tc-1)-tl(tc-1)) + tl0$tfirst(tc);
	teref(tc) = te(tc););
$offecho

*       Generate GAMS code for numerical differencing

$onecho >jacobian.gms
	eref(t) = E.L(t);
	pdref(t) = PD.L(t);
	LOOP((tlast,tc)$(not t(tc)),
	  eref(tc)  = eref(tlast)  * (sigma(tc)*L(tc))/(sigma(tlast)*L(tlast));
	  pdref(tc) = pdref(tlast) * (L(tc)*rr(tc))   /(L(tlast)*rr(tlast));
	);
$include climatemodel
	teinit(tc)  = teref(tc);
	grad(tc,tp) = 0;	
	loop(tp, eref(tp) = eref(tp) + deltaE;
$include climatemodel
	  grad(tc,tp) = (teref(tc)-teinit(tc)) / deltaE;
	  eref(tp)    = eref(tp) - deltaE;);
	teref(tc)     = teinit(tc);
	loop(tlast,
	  xi(t) = sum(tc$(not t(tc)), grad(tc,t)*2*(a1/9)*(teref(tc) 
	     + sum(tpp, grad(tc,tpp)*(E.L(tpp)-eref(tpp))))*pdref(tc)) / pdref(tlast);
	);

$offecho

K.L(t) = k0*0.9 * L(t)/sum(tfirst,L(tfirst));
Q.L(t) = al(t) * L(t)**(1-gamma) * K.L(t)**gamma;
I.L(t) = (K.L(t+1) - (1-dk)**10*K.L(t)) / 10;
MIU.l(t) = 0.1;
A.L(t) =  b1 * MIU.L(t)**b2;
D.L(t) = 0;
Y.L(t) = Q.L(t)*(1-A.L(t))/ (1 + D.L(t));
C.l(t) = Y.L(t) - I.L(t);
PC.L(t) = 10 * rr(t) * L(t) / (0.55*C.L(t));
PY.L(t) = PC.L(t) / (1+D.L(t));
PQ.l(t) = PY.l(t);
PK.l(t) = PY.l(t);
PA.l(t) = -PY.L(t)*Q.L(t);
PD.l(t) = -PY.L(t)*Y.L(t);
PE.l(t) = -PA.L(t)*b1*b2*MIU.L(t)**(b2-1)/(10*sigma(t)*Q.L(t));
MIU.UP(t) = 0.99;
MIU.LO(t) = 0.01;
KT.L = sum(tlast, K.L(tlast));
PKT.L = sum(tlast, PK.L(tlast)); PKT.UP = +INF;
TK.FX = tk0;
E.L(t) = 10 * sigma(t) * 0.9 * Q.L(t);

set	diagitr	Diagonalization iterations /iter0*iter4/;

LOOP(diagitr,
$INCLUDE jacobian
	SOLVE DICEMCP USING MCP;
);
$INCLUDE jacobian

*	Now endogenize the capital tax rate and consider look at 
*	the marginal utility of emission reductions:

PARAMETER	u0	Reference utility,
		du(t)	Marginal utility of emission reductions;


g0 = G.L;
TK.UP = +inf;
TK.LO = -inf;
E.fx(t) = E.L(t);

alias (t,tloop);
loop(tc$(not t(tc)),  C.L(tc) = C.L(tc-1) * L(tc)/L(tc-1););
u0 = sum(tc, 10 * rr(tc)*L(tc)*LOG(C.l(tc)/L(tc))/0.55);

LOOP(tloop,
	E.FX(tloop) = E.L(tloop)/1.01;
	SOLVE DICEMCP USING MCP;
	E.FX(tloop) = E.L(tloop)*1.01;
	du(tloop) = sum(tc, 10 * rr(tc)*L(tc)*LOG(C.l(tc)/L(tc))/0.55) - u0;
);
