$title	DICE version 1994 -- with cosmetic revisions

$include dicedata

alias (t,tp);

PARAMETER
	teref(t)	Reference values of temperature,
	grad(tp,t)	Local dependence of TE(tp) on E(t),
	eref(t)		Reference values of emissions,
	gradt(tp)	Local dependence of transversality terms on emissions;

VARIABLES
	C(t)		Consumption trillion US dollars
	K(t)		Capital stock trillion US dollars
	I(t)		Investment trillion US dollars
	D(t)		Damage
	A(t)		Abatement cost
	Y(t)		Output net abatement and damage costs 
	Q(t)		Gross Output 
	E(t)		CO2-equiv emissions billion t
	MIU(t)		Emission control rate GHGs
	UTILITY		Maximand;

POSITIVE VARIABLES MIU, E, Y, C, K, I;

EQUATIONS
	UTIL		Objective function
	YY(t)		Output
	AA(t)		Abatement
	QQ(t)		Underlying production function
	CC(t)		Consumption
	KK(t)		Capital balance
	KC(t)		Terminal condition of K
	EE(t)		Emissions process,
	DD(t)		Damage ;

CC(t)..		C(t) =E= Y(t) - I(t);
YY(t)..		Y(t) =E= Q(t) - A(t)*Q(t) - D(t)*Y(t);
AA(t)..		A(t) =E=  b1 * MIU(t)**b2;
QQ(t)..		Q(t) =E= al(t) * L(t)**(1-gamma) * K(t)**gamma;
KK(t)..		K(t) =L= (1-dk)**10 * K(t-1) + 10 * I(t-1) + (0.9*k0)$tfirst(t);
KC(tlast)..	dk * K(tlast) =L= I(tlast);
EE(t)..		E(t) =G= 10 * sigma(t) * (1-MIU(t)) * Q(t);
DD(t)..		D(t) =E= (a1/9)*SQR(teref(t) + sum(tp, grad(t,tp)*(E(tp)-eref(tp))));
UTIL..		UTILITY =E= SUM(t, 10 *rr(t)*L(t)*LOG(C(t)/L(t))/0.55)
	 + SUM(tlast, rr(tlast)*((phik*K(tlast))+sum(t,gradt(t)*(E(t)-eref(t)))));

*	Assign a naive starting point which is in the 
*	domain of the functions:

K.L(t) = k0 * L(t)/L0;
K.LO(t) = K.L(t)/100;
Q.L(t)	= al(t) * K.L(t)**gamma * L(t)**(1-gamma);
MIU.L(t) = 0;
E.L(t) = 10 * sigma(t) * (1-MIU.L(t)) * Q.L(t);
Y.L(t) = Q.L(t);
C.L(t) = 1;
A.L(t) = 0;
D.L(t) = 0;
I.L(t) = dk*K.L(T)/10;
MIU.L(t) = 1;
UTILITY.L = 1;

* Upper and Lower Bounds for economic reasons or stability

MIU.UP(t) = 0.99;
MIU.LO(t) = 0.01;
K.LO(t) = 1;
C.LO(t) = 2;

*	Initial values:

MIU.fx('1')=0;
MIU.fx('2')=0;
MIU.fx('3')=0;

MODEL dice /all/;

parameters
	m(t)		CO2-equiv concentration billion t
	forc(t)		Radiative forcing - W per m2
	te(t)		Temperature - atmosphere C
	tl(t)		Temperature - lower ocean C
	termv		Terminal value of atmophere
	deltaE		Difference iterval /0.001/;

m(tfirst) = M0;
te(tfirst) = T0;
tl(tfirst) = TL0;

parameter  teinit(t)	Tracking of initial temperature trajectory;


*-------------------------------------------------------------------------------
m(t) = m0;
te(t) = t0;
tl(t) = tl0;
$onecho >climatemodel.gms
loop(t,
	m(t)		= 590 + atret*eref(t) + (1-deltam)*(m(t-1)-590) + m0$tfirst(t);
	forc(t)		= 4.1*(LOG(m(t)/590)/LOG(2)) + forcoth(t);
	te(t)		= te(t-1)+c1*(forc(t-1)-lam*te(t-1)-c3*(te(t-1)-tl(t-1))) + t0$tfirst(t);
	tl(t)		= tl(t-1)+c4*(te(t-1)-tl(t-1)) + tl0$tfirst(t);
	teref(t)	= te(t);
);
$offecho

$onecho >jacobian.gms
	eref(t) = E.L(t);
$include climatemodel
	teinit(t) = teref(t); termv  = sum(tlast, phim*m(tlast)+phite*te(tlast));
	grad(t,tp) = 0;	gradt(t) = 0;
	loop(tp,eref(tp) = eref(tp) + deltaE;
$include climatemodel
	  grad(t,tp)= (teref(t)-teinit(t)) / deltaE;
	  gradt(tp) = (sum(tlast, phim*m(tlast)+phite*te(tlast))-termv) / deltaE;
	  eref(tp)  = eref(tp) - deltaE;);
	  teref(t)  = teinit(t);
$offecho

set	diagitr		Diagonalization iterations /diag1*diag5/;
loop(diagitr,
	eref(t) = e.l(t);
$include jacobian
	SOLVE dice maximizing UTILITY using NLP;
);

execute_unload 'nlpdecompsol.gdx',eref,C,K,I,D,A,Y,Q,E,MIU,YY,AA,QQ,CC,KK,KC,EE,DD;
